#!/bin/bash

export LANGUAGE=en_US.UTF-8
export LANG=en_US.UTF-8
export LC_ALL=en_US.UTF-8

if [ -f /proc/user_beancounters ]; then
  /bin/echo "This is an OpenVZ Container. Network settings may not be changed from inside the VPS."
  exit
fi

: ${DIALOG=dialog}

# Start: Pause to wait for cced constructors to finish
WAITFOR=cce_construct
RESULT=`pgrep -f $WAITFOR|wc -l`
while [ "$RESULT" -gt "0" ]; do
  sleep 1
  echo "75" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nPlease wait for CCEd Constructors to finish ..." 10 70 0
  sleep 3
  RESULT=`pgrep -f $WAITFOR|wc -l`
done
echo "100" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nPlease wait for CCEd Constructors to finish ..." 10 70 0
sleep 3
# End: Pause to wait for cced constructors to finish

SERVERNAME=`uname -n|head -1`
IPADDRESS=`ifconfig  | grep 'inet '| grep -v '127.0.0.1' | cut -d: -f2 | awk '{ print $2}'|head -1`
NETMASK=`ifconfig | grep $IPADDRESS | awk '{ print $4}'`
DEFAULTGW=`/sbin/ip route | awk '/default/ { print $3 }'|head -1`
DNSSERVER=`cat /etc/resolv.conf |grep ^nameserver|awk '{ print $2}'|head -1`
IPV6ADDRESS=`/sbin/ip -6 -o addr show |grep eth0|grep -v fe80|awk '{print$4}'| tr '/' ' ' | awk '{print$1}'|head -1`
DEFAULTGWV6=`/sbin/ip -6 route show default|grep -v fe80|awk '{print$3}'|head -1`

function GetServerName() {
MSG="Please Enter the Fully Qualified Domain Name of your Server (FQDN)\n\n"
exec 3>&1
SERVERNAME="`$DIALOG --nocancel --backtitle "$CDTITLE" --title "$TITLE"  \
  --inputbox "$MSG" 15 70 $SERVERNAME 2>&1 1>&3`"
retval=$?
exec 3>&-
RESET=0
}

function GetIP() {
MSG="Please Enter your IPv4 IP Address\n\n"
exec 3>&1
IPADDRESS="`$DIALOG --nocancel --backtitle "$CDTITLE" --title "$TITLE"  \
  --inputbox "$MSG" 15 70 $IPADDRESS 2>&1 1>&3`"
retval=$?
exec 3>&-
RESET=0
}

function GetNM() {
MSG="Please Enter your Netmask\n\n"
exec 3>&1
NETMASK="`$DIALOG --nocancel --backtitle "$CDTITLE" --title "$TITLE"  \
  --inputbox "$MSG" 15 70 $NETMASK 2>&1 1>&3`"
retval=$?
exec 3>&-
RESET=0
}

function GetGW() {
MSG="Please Enter your Default IPv4 Gateway\n\n"
exec 3>&1
DEFAULTGW="`$DIALOG --nocancel --backtitle "$CDTITLE" --title "$TITLE"  \
  --inputbox "$MSG" 15 70 $DEFAULTGW 2>&1 1>&3`"
retval=$?
exec 3>&-
RESET=0
}

function GetDNS() {
MSG="Please Enter your DNS Server IP\n\n"
exec 3>&1
DNSSERVER="`$DIALOG --nocancel --backtitle "$CDTITLE" --title "$TITLE"  \
  --inputbox "$MSG" 15 70 $DNSSERVER 2>&1 1>&3`"
retval=$?
exec 3>&-
RESET=0
}

function GetIPv6() {
MSG="Please Enter your IPv6 IP Address\n\n"
exec 3>&1
IPV6ADDRESS="`$DIALOG --nocancel --backtitle "$CDTITLE" --title "$TITLE"  \
  --inputbox "$MSG" 15 70 $IPV6ADDRESS 2>&1 1>&3`"
retval=$?
exec 3>&-
RESET=0
}

function GetV6GW() {
MSG="Please Enter your Default IPv6 Gateway\n\n"
exec 3>&1
DEFAULTGWV6="`$DIALOG --nocancel --backtitle "$CDTITLE" --title "$TITLE"  \
  --inputbox "$MSG" 15 70 $DEFAULTGWV6 2>&1 1>&3`"
retval=$?
exec 3>&-
RESET=0
}

CDTITLE="Team BlueOnyx Presents - Network Reconfigure"
TITLE="Network Setup Utility"

IPv4=3
IPv6=3
V4=0
V6=0

function AskIPv4 {
        MSG="Do you want to configure IPv4?"
        $DIALOG --nocancel --backtitle "$CDTITLE" --title "$TITLE" \
          --yesno "$MSG" 0 0
        IPv4=$?
}

if [ "$IPv4" == "3" ]; then
        AskIPv4
fi
if [ "$IPv4" -eq "0" ]; then
  echo "IPv4 setup ..."
  GetServerName
  GetIP
  GetNM
  GetGW
  GetDNS
fi
if [ "$IPv4" -eq "1" ]; then
  echo "Skipping IPv4 setup"
fi

function AskIPv6 {
        MSG="Do you want to configure IPv6?"
        $DIALOG --nocancel --backtitle "$CDTITLE" --title "$TITLE" \
          --yesno "$MSG" 0 0
        IPv6=$?
}

if [ "$IPv6" == "3" ]; then
        AskIPv6
fi
if [ "$IPv6" -eq "0" ]; then
  echo "IPv6 setup ..."
  GetServerName
  GetIPv6
  GetV6GW
  GetDNS
fi
if [ "$IPv6" -eq "1" ]; then
  echo "Skipping IPv6 setup"
fi

if [ "$IPv6" -eq "1" ] && [ "$IPv4" -eq "1" ];then
  echo "Not configuring network at all."
  exit
fi

function Confirm() {
MSG="
Please Confirm your Network Settings\n\n
Server Name  : $SERVERNAME\n
IPv4 Address : $IPADDRESS\n
Netmask      : $NETMASK\n
Gateway      : $DEFAULTGW\n
DNS Server   : $DNSSERVER\n
IPv6 Address : $IPV6ADDRESS\n
IPv6 Gateway : $DEFAULTGWV6\n
\n
Accept?"
$DIALOG --nocancel --backtitle "$CDTITLE" --title "$TITLE" \
  --yesno "$MSG" 0 0
NO=$?
}

Confirm
if [ "$NO" -eq "1" ];then
  echo "Exiting without applying any changes."
  exit
fi

GOTSERVERNAME=0;
if [ "$SERVERNAME" != "" ];then
  GOTSERVERNAME=1;
else
  SERVERNAME='localhost.localdomain';
fi

GOTIPv4=0;
if [ "$IPADDRESS" != "" ] && [ "$NETMASK" != "" ] && [ "$DEFAULTGW" != "" ];then
  GOTIPv4=1;
  V4=1
fi
GOTIPv6=0;
if [ "$IPV6ADDRESS" != "" ] && [ "$DEFAULTGWV6" != "" ];then
  GOTIPv6=1;
  V6=1
fi

if [ "$GOTIPv4" -eq "1" ] || [ "$GOTIPv6" -eq "1" ]; then

  if [ "$SERVERNAME" != "" ];then
      # And we're really going to town with this:
      /usr/sausalito/sbin/set_servername.pl configure $SERVERNAME >/dev/null 2>&1 || :
  fi

  # Change the default gateway
  /usr/bin/perl -pi -e "s#GATEWAY=.*#GATEWAY=$DEFAULTGW#" /etc/sysconfig/network

  # Find the Network ID & Broadcast ID
  NETWORKID=`/usr/sausalito/sbin/minicalc.pl id $IPADDRESS $NETMASK`
  BROADCAST=`/usr/sausalito/sbin/minicalc.pl bid $IPADDRESS $NETMASK`

  # Create/Update /etc/udev/rules.d/70-persistent-net.rules:
  if [ ! -f /etc/udev/rules.d/70-persistent-net.rules ]; then
    /usr/sausalito/sbin/write_udev.pl > /etc/udev/rules.d/70-persistent-net.rules
    # Let's get rid of ifcfg-* files for non-ethX interfaces:
    /bin/cp /etc/sysconfig/network-scripts/ifcfg-* /tmp/
    /bin/rm -f /etc/sysconfig/network-scripts/ifcfg-*
    /bin/cp /tmp/ifcfg-lo /etc/sysconfig/network-scripts/
  fi

  # Configure /etc/sysconfig/network:

  /bin/echo "FORWARD_IPV4=false" > /etc/sysconfig/network
  if [ "$GOTIPv4" -eq "1" ] && [ "$DEFAULTGW" != "" ];then
    /bin/echo "GATEWAY=$DEFAULTGW" >> /etc/sysconfig/network
    /bin/echo "NETWORKING=yes" >> /etc/sysconfig/network
  fi
  /bin/echo "HOSTNAME=$SERVERNAME" >> /etc/sysconfig/network
  /bin/echo "IPV6_AUTOCONF=no" >> /etc/sysconfig/network
  if [ "$GOTIPv6" -eq "1" ] && [ $DEFAULTGWV6 != "" ];then
    /bin/echo "IPV6FORWARDING=yes" >> /etc/sysconfig/network
    /bin/echo "IPV6_DEFAULTDEV=eth0" >> /etc/sysconfig/network
    /bin/echo "IPV6_DEFAULTGW=$DEFAULTGWV6" >> /etc/sysconfig/network
    /bin/echo "NETWORKING_IPV6=yes" >> /etc/sysconfig/network
    sleep 5
  fi
  /bin/echo "NOZEROCONF=yes" >> /etc/sysconfig/network

  # Configure up the init scripts for eth0 properly.
  /bin/echo "DEVICE=eth0" > /etc/sysconfig/network-scripts/ifcfg-eth0
  /bin/echo "BOOTPROTO=none" >> /etc/sysconfig/network-scripts/ifcfg-eth0
  /bin/echo "ONBOOT=yes" >> /etc/sysconfig/network-scripts/ifcfg-eth0
  /bin/echo "DELAY=0" >> /etc/sysconfig/network-scripts/ifcfg-eth0
  /bin/echo "NM_CONTROLLED=yes" >> /etc/sysconfig/network-scripts/ifcfg-eth0
  /bin/echo "DNS0=$DNSSERVER" >> /etc/sysconfig/network-scripts/ifcfg-eth0

  if [ "$GOTIPv4" -eq "1" ];then
    /bin/echo "BROADCAST=$BROADCAST" >> /etc/sysconfig/network-scripts/ifcfg-eth0
    /bin/echo "NETWORK=$NETWORKID" >> /etc/sysconfig/network-scripts/ifcfg-eth0
    /bin/echo "NETMASK=$NETMASK" >> /etc/sysconfig/network-scripts/ifcfg-eth0
    /bin/echo "IPADDR=$IPADDRESS" >> /etc/sysconfig/network-scripts/ifcfg-eth0
  fi
  /bin/echo "USERCTL=no" >> /etc/sysconfig/network-scripts/ifcfg-eth0
  /bin/echo "ARPCHECK=no" >> /etc/sysconfig/network-scripts/ifcfg-eth0
  if [ "$GOTIPv6" -eq "1" ];then
    /bin/echo "IPV6INIT=yes" >> /etc/sysconfig/network-scripts/ifcfg-eth0
    /bin/echo "IPV6ADDR=$IPV6ADDRESS" >> /etc/sysconfig/network-scripts/ifcfg-eth0
    /bin/echo "IPV6_DEFAULTGW=$DEFAULTGWV6" >> /etc/sysconfig/network-scripts/ifcfg-eth0
    sleep 5
  fi

  # At this time during initial startup on first-boot Systemd has Rsyslog throw a hissie-fit. So we restart Rsyslog:
  systemctl restart rsyslog &>/dev/null || :

  # Now do it the proper way via CCEd SET transaction:
  if [ "$V4" -eq "1" ] && [ "$V6" -eq "1" ]; then
    #echo "/usr/sausalito/sbin/nethelper.pl --device=eth0 --ipv4ip=$IPADDRESS --ipv4nm=$NETMASK --ipv4gw=$DEFAULTGW --ipv6=$IPV6ADDRESS --ipv6gw=$DEFAULTGWV6 --dns=$DNSSERVER"
    /usr/sausalito/sbin/nethelper.pl --device=eth0 --ipv4ip=$IPADDRESS --ipv4nm=$NETMASK --ipv4gw=$DEFAULTGW --ipv6=$IPV6ADDRESS --ipv6gw=$DEFAULTGWV6 --dns=$DNSSERVER
  fi
  if [ "$V4" -eq "1" ] && [ "$V6" -eq "0" ]; then
    #echo "/usr/sausalito/sbin/nethelper.pl --device=eth0 --ipv4ip=$IPADDRESS --ipv4nm=$NETMASK --ipv4gw=$DEFAULTGW --dns=$DNSSERVER"
    /usr/sausalito/sbin/nethelper.pl --device=eth0 --ipv4ip=$IPADDRESS --ipv4nm=$NETMASK --ipv4gw=$DEFAULTGW --dns=$DNSSERVER
  fi
  if [ "$V4" -eq "0" ] && [ "$V6" -eq "1" ]; then
    #echo "/usr/sausalito/sbin/nethelper.pl --device=eth0 --ipv6=$IPV6ADDRESS --ipv6gw=$DEFAULTGWV6 --dns=$DNSSERVER"
    /usr/sausalito/sbin/nethelper.pl --device=eth0 --ipv6=$IPV6ADDRESS --ipv6gw=$DEFAULTGWV6 --dns=$DNSSERVER
  fi

  # Restart Network:
  echo percentage | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "text" height width percent
  echo "10" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  systemctl enable NetworkManager.service &>/dev/null || :
  echo "20" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  systemctl restart NetworkManager.service --no-block &>/dev/null || :
  echo "30" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  systemctl enable swatch.service &>/dev/null || :
  echo "40" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  /usr/sausalito/constructor/base/network/10_fix_ifup.pl
  echo "50" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  /usr/sausalito/constructor/base/network/30_addNetwork.pl
  echo "60" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  /usr/sausalito/constructor/base/network/40_addGateway.pl
  echo "70" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  /usr/sausalito/constructor/base/network/checkInterfaceConfigure.pl
  echo "75" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  # Convince Sausalito that we are using eth0 & have proper IP info already
  /usr/sausalito/sbin/set_eth.pl configure eth0 $IPADDRESS $NETMASK
  echo "80" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  /usr/sausalito/sbin/set_gw.pl configure $DEFAULTGW
  echo "90" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0
  /usr/sausalito/sbin/set_dns.pl configure $DNSSERVER
  # Remove all non-ethX style interfaces except for lo:
  ls -k1 /etc/sysconfig/network-scripts/ifcfg-*|grep -v ifcfg-lo|grep -v ifcfg-eth|xargs rm -f
  /usr/sbin/ifup eth0 &>/dev/null || :
  echo "100" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nConfiguring Network" 10 70 0

  # restart daemons
  echo percentage | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "text" height width percent
  echo "0" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nRestarting Daemons" 10 70 0
  systemctl restart httpd > /dev/null 2>&1 || :
  echo "25" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nRestarting Daemons" 10 70 0
  systemctl restart admserv > /dev/null 2>&1 || :
  echo "50" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nRestarting Daemons" 10 70 0
  systemctl restart sendmail > /dev/null 2>&1 || :
  echo "75" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nRestarting Daemons" 10 70 0
  systemctl restart admserv > /dev/null 2>&1 || :
  echo "90" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nRestarting Daemons" 10 70 0
  systemctl restart firewalld > /dev/null 2>&1 || :
  echo "100" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nRestarting Daemons" 10 70 0

  # Turn off the EL8 firewall:
  #systemctl stop firewalld.service --no-block &>/dev/null || :
  #systemctl disable firewalld.service &>/dev/null || :
  #rm -f /etc/systemd/system/dbus-org.fedoraproject.FirewallD1.service
  #rm -f /etc/systemd/system/basic.target.wants/firewalld.service

  # CCEd restart:
  echo percentage | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "text" height width percent
  echo "10" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nRehashing CCed" 10 70 0
  echo "45" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nRehashing CCed" 10 70 0
  /usr/sausalito/sbin/cced.init rehash &>/dev/null || :
  echo "100" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nRehashing CCed" 10 70 0
  
  # Run cce_construct_dialog for the constructors:
  /usr/sausalito/sbin/cce_construct_dialog

  # Run Swatch:
  echo percentage | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "text" height width percent
  echo "10" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nRunning Swatch to make sure all services are up" 10 70 0
  echo "45" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nRunning Swatch to make sure all services are up" 10 70 0
  /usr/sausalito/sbin/swatch.sh &>/dev/null || :
  echo "100" | dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --gauge "BlueOnyx 5211R initial setup:\nRunning Swatch to make sure all services are up" 10 70 0

  # Configure /etc/issue:
  /bin/echo "BlueOnyx 5211R on \S" > /etc/issue
  /bin/echo "Kernel \r on an \m" >> /etc/issue
  /bin/echo "" >> /etc/issue
  if [ "$V4" -eq "1" ];then
          /bin/echo "IPv4 Address: $IPADDRESS" >> /etc/issue
  fi   
  if [ "$V6" -eq "1" ];then
          /bin/echo "IPv6 Address: $IPV6ADDRESS" >> /etc/issue
  fi
  /bin/echo "" >> /etc/issue

  # Clean up /root/.bashrc:
  HAVEBASHRC=$(cat /root/.bashrc|grep "network_settings.sh"|wc -l)
  if [ "$HAVEBASHRC" -gt "0" ];then
    /bin/echo "# Source global definitions" > /root/.bashrc
    /bin/echo "if [ -f /etc/bashrc ]; then" >> /root/.bashrc
    /bin/echo "        . /etc/bashrc" >> /root/.bashrc
    /bin/echo "fi" >> /root/.bashrc
  fi

  # Run serverNameGui.pl:
  if [ -f /usr/sausalito/constructor/base/wizard/serverNameGui.pl ];then
    /usr/sausalito/constructor/base/wizard/serverNameGui.pl >/dev/null 2>&1 || :
  fi

  # Run Swatch in background, suppressing all output:
  /usr/sbin/swatch > /dev/null 2>&1 &

  # End-Message:
  echo "" > /tmp/.bxgreeting
  echo "Network Setup of BlueOnyx 5211R has been completed." >> /tmp/.bxgreeting
  echo "=========================================================" >> /tmp/.bxgreeting
  echo "" >> /tmp/.bxgreeting
  echo "You can reach the GUI at these two URL's in your browser:" >> /tmp/.bxgreeting
  echo "" >> /tmp/.bxgreeting
  echo "http://$IPADDRESS:444/login" >> /tmp/.bxgreeting
  echo "" >> /tmp/.bxgreeting
  echo "https://$IPADDRESS:81/login" >> /tmp/.bxgreeting
  echo "" >> /tmp/.bxgreeting
  echo "Enjoy BlueOnyx 5211R!" >> /tmp/.bxgreeting
  echo "" >> /tmp/.bxgreeting
  dialog --nocancel --backtitle "$CDTITLE" --title "$TITLE" --textbox /tmp/.bxgreeting 22 70
  rm -f /tmp/.bxgreeting

fi

# 
# Copyright (c) 2019-2024 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2019-2024 Team BlueOnyx, BLUEONYX.IT
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#   notice, this list of conditions and the following disclaimer in 
#   the documentation and/or other materials provided with the 
#   distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#   contributors may be used to endorse or promote products derived 
#   from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 
